@@ -6,15 +6,16 @@
 	 * http://www.banshee-php.org/
 	 */
 
+
 	class XML {
 		private $xml_header = "<?xml version=\"1.0\" encoding=\"utf-8\"?>";
-		private $db = null;
 		private $xml_data = "";
 		private $xslt_parameters = array();
 		private $open_tags = array();
-		private $cache_id = null;
+		private $cache = null;
+		private $cache_key = null;
 		private $cache_buffer = "";
-		private $cache_timeout;
+		private $tag_eol = true;
 
 		/* Constructor
 		 *
@@ -23,7 +24,9 @@
 		 * ERROR:  -
 		 */
 		public function __construct($db = null) {
-			$this->db = $db;
+			if ($db !== null) {
+				$this->cache = new cache($db, "xml");
+			}
 		}
 
 		/* Magic method get
@@ -37,11 +40,23 @@
 				case "depth": return count($this->open_tags);
 				case "data": return $this->xml_data;
 				case "document": return $this->xml_header."\n".$this->xml_data;
+				case "array": return $this->xml_to_array($this->xml_data);
 			}
 
 			return null;
 		}
 
+		/* Clear XML data buffer
+		 *
+		 * INPUT:  -
+		 * OUTPUT: -
+		 * ERROR:  -
+		 */
+		public function clear_buffer() {
+			$this->xml_data = "";
+			$this->abort_caching();
+		}
+
 		/* Translate special characters in string to XML entities
 		 *
 		 * INPUT:  string data
@@ -64,25 +79,25 @@
 		private function add_to_buffer($str) {
 			$this->xml_data .= $str;
 
-			if ($this->cache_id !== null) {
+			if ($this->cache_key !== null) {
 				$this->cache_buffer .= $str;
 			}
 		}
 
 		/* Add open-tag to buffer
 		 *
-		 * INPUT:  string tag, array( string data[, ...] )[, boolean append EOL]
+		 * INPUT:  string tag name, array( string attributes[, ...] )
 		 * OUTPUT: -
 		 * ERROR:  -
 		 */
-		public function open_tag($tag, $params = array(), $eol = true) {
-			$this->add_to_buffer("<".$tag);
-			foreach ($params as $key => $value) {
+		public function open_tag($name, $attributes = array()) {
+			$this->add_to_buffer("<".$name);
+			foreach ($attributes as $key => $value) {
 				$this->add_to_buffer(" ".$key."=\"".$this->xmlspecialchars($value)."\"");
 			}
-			$this->add_to_buffer(">".($eol ? "\n" : ""));
+			$this->add_to_buffer(">".($this->tag_eol ? "\n" : ""));
 
-			array_push($this->open_tags, $tag);
+			array_push($this->open_tags, $name);
 		}
 
 		/* Add close-tag to buffer
@@ -101,36 +116,53 @@
 
 		/* Add tag to buffer
 		 *
-		 * INPUT:  string tag, string data, array( string key => string value[, ...] )
+		 * INPUT:  string tag name, string data, array( string key => string value[, ...] )
 		 * OUTPUT: -
 		 * ERROR:  -
 		 */
-		public function add_tag($tag, $data = null, $params = array()) {
-			$this->open_tag($tag, $params, false);
+		public function add_tag($name, $data = null, $attributes = array()) {
+			$this->tag_eol = false;
+
+			$this->open_tag($name, $attributes);
 			if ($data !== null) {
 				$this->add_to_buffer($this->xmlspecialchars($data));
 			}
 			$this->close_tag();
+
+			$this->tag_eol = true;
 		}
 
 		/* Add record to buffer
 		 *
-		 * INPUT:  array( string key => string value[, ...] ), string tag name, array( string key => string value[, ...] )
+		 * INPUT:  array( string key => string value[, ...] )[, string tag name][, array( string key => string value[, ...] )][, boolean recursive]
 		 * OUTPUT: -
 		 * ERROR:  -
 		 */
-		public function record($record, $name = null, $params = array()) {
+		public function record($record, $name = null, $attributes = array(), $recursive = false) {
 			if ($name !== null) {
 				if (isset($record["id"])) {
-					$params["id"] = $record["id"];
+					$attributes["id"] = $record["id"];
 				}
-				$this->open_tag($name, $params);
+
+				if (is_numeric($name)) {
+					$name = "item";
+				}
+				$this->open_tag($name, $attributes);
 			}
 
 			$skip_tags = array("id", "password");
 			foreach (array_keys($record) as $key) {
-				if ((in_array($key, $skip_tags) == false) && (is_array($record[$key]) == false)) {
+				if (in_array($key, $skip_tags, true)) {
+					continue;
+				}
+				
+				if (is_array($record[$key]) == false) {
+					if (is_numeric($key)) {
+						$key = "item";
+					}
 					$this->add_tag($key, $record[$key]);
+				} else if ($recursive) {
+					$this->record($record[$key], $key, array(), true);
 				}
 			}
 			if ($name !== null) {
@@ -150,7 +182,9 @@
 			}
 
 			if (($begin = strpos($str, "<".$tag.">")) === false) {
-				return;
+				if (($begin = strpos($str, "<".$tag." ")) === false) {
+					return;
+				}
 			}
 			if (($end = strrpos($str, "</".$tag.">")) === false) {
 				return;
@@ -172,6 +206,59 @@
 			$this->xslt_parameters[$key] = $value;
 		}
 
+		/* Convert XML string to array
+		 *
+		 * INPUT:  string xml
+		 * OUTPUT: array data
+		 * ERROR:  false
+		 */
+		public function xml_to_array($xml) {
+			/* Convert XML to array
+			 */
+			$parser = xml_parser_create();
+			xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, 0);
+			if (xml_parse_into_struct($parser, $xml, $items, $index) == 0) {
+				return false;
+			}
+			xml_parser_free($parser);
+
+			/* Convert to usable array
+			 */
+			$result = $pointers = array();
+			$depth = 0;
+			$pointers[$depth] = &$result;
+
+			foreach ($items as $item) {
+				if ($item["attributes"] === null) {
+					$item["attributes"] = array();
+				}
+
+				/* Handle node types
+				 */
+				switch ($item["type"]) {
+					case "open":
+						array_push($pointers[$depth], array(
+							"name"       => $item["tag"],
+							"attributes" => $item["attributes"],
+							"content"    => array()));
+						$last = count($pointers[$depth]) - 1;
+						$pointers[$depth + 1] = &$pointers[$depth++][$last]["content"];
+						break;
+					case "complete":
+						array_push($pointers[$depth], array(
+							"name"       => $item["tag"],
+							"attributes" => $item["attributes"],
+							"content"    => trim($item["value"])));
+						break;
+					case "close":
+						$depth--;
+						break;
+				}
+			}
+
+			return $result;
+		}
+
 		/* Perform XSL transformation
 		 *
 		 * INPUT:  string XSLT filename
@@ -179,68 +266,59 @@
 		 * ERROR:  false
 		 */
 		public function transform($xslt_file) {
-			$xslt = new DomDocument();
-			$xml = new DomDocument();
+			$xslt = new domdocument;
+			$xml = new domdocument;
 
-			if ($xslt->load("../views/".$xslt_file.".xslt") == false) {
+			if ((file_exists($xslt_file)) == false) {
 				return false;
-			}
-			
-			if ($xml->loadXML($this->document) == false) {
+			} else if ($xslt->load($xslt_file) == false) {
 				return false;
 			}
 
-			$processor = new XsltProcessor();
-			$processor->importStylesheet($xslt);
+			if ($xml->loadxml($this->document) == false) {
+				return false;
+			}
+
+			$processor = new xsltprocessor();
+			$processor->importstylesheet($xslt);
 
 			foreach ($this->xslt_parameters as $key => $value) {
-				$processor->setParameter("", $key, $value);
+				$processor->setparameter("", $key, $value);
 			}
 
-			return $processor->transformToXML($xml);
+			return $processor->transformtoxml($xml);
 		}
 
 		/* Start caching XML additions done after this call
 		 *
-		 * INPUT:  string cache identifier[, int cache timeout in seconds]
-		 * OUTPUT: boolean start caching successful
-		 * ERROR:  -
+		 * INPUT:  string cache key
+		 * OUTPUT: true
+		 * ERROR:  false
 		 */
-		public function start_caching($id, $timeout = CACHE_TIMEOUT) {
-			if (($this->db === null) || ($this->cache_id !== null)) {
+		public function start_caching($key) {
+			if (($this->cache === null) || ($this->cache_key !== null)) {
 				return false;
 			}
 
-			$this->cache_id = $id;
-			$this->cache_timeout = date("Y-m-d H:i:s", strtotime("+".$timeout." seconds"));
+			$this->cache_key = $key;
 
 			return true;
 		}
 
-		/* Stop caching
+		/* Stop XML caching
 		 *
 		 * INPUT:  -
-		 * OUTPUT: boolean stop caching successfull
-		 * ERROR:  -
+		 * OUTPUT: true
+		 * ERROR:  false
 		 */
 		public function stop_caching() {
-			if (($this->db === null) || ($this->cache_id === null)) {
+			if (($this->cache === null) || ($this->cache_key === null)) {
 				return false;
 			}
 
-			if ($this->remove_from_cache($this->cache_id) === false) {
-				return false;
-			}
-
-			$data = array(
-				"id"      => $this->cache_id,
-				"content" => $this->cache_buffer,
-				"timeout" => $this->cache_timeout);
-			if (($this->db->insert("cache", $data)) == false) {
-				return false;
-			}
-
-			$this->abort_caching();
+			$this->cache->{$this->cache_key} = $this->cache_buffer;
+			$this->cache_key = null;
+			$this->cache_buffer = "";
 
 			return true;
 		}
@@ -248,63 +326,54 @@
 		/* Abort XML caching
 		 *
 		 * INPUT:  -
-		 * OUTPUT: -
-		 * ERROR:  -
+		 * OUTPUT: true
+		 * ERROR:  false
 		 */
-		public function abort_caching() {	
-			$this->cache_id = null;
+		public function abort_caching() {
+			if ($this->cache === null) {
+				return false;
+			}
+
+			$this->cache_key = null;
 			$this->cache_buffer = "";
+
+			return true;
 		}
 
 		/* Fetch XML from cache and append to buffer
 		 *
-		 * INPUT:  string cache identifier
-		 * OUTPUT: -
-		 * ERROR:  -
+		 * INPUT:  string cache key
+		 * OUTPUT: true
+		 * ERROR:  false
 		 */
-		public function fetch_from_cache($id) {
-			if ($this->db === null) {
+		public function fetch_from_cache($key) {
+			if ($this->cache === null) {
 				return false;
 			}
 
-			$query = "select * from cache where id=%s and timeout>now() limit 1";
-			if (($result = $this->db->execute($query, $id)) == false) {	
+			if (($value = $this->cache->$key) == null) {
 				return false;
 			}
 
-			$this->xml_data .= $result[0]["content"];
+			$this->xml_data .= $value;
 
 			return true;
 		}
 
 		/* Remote entry from XML cache
 		 *
-		 * INPUT:  string cache identifier
-		 * OUTPUT: boolean removal successful
-		 * ERROR:  -
+		 * INPUT:  string cache key
+		 * OUTPUT: true
+		 * ERROR:  false
 		 */
-		public function remove_from_cache($id) {
-			if ($this->db === null) {
+		public function remove_from_cache($key) {
+			if ($this->cache === null) {
 				return false;
 			}
 
-			$query = "delete from cache where id=%s";
-			return $this->db->query($query, $id) !== false;
-		}
+			$this->cache->$key = null;
 
-		/* Clear entire cache
-		 *
-		 * INPUT:  -
-		 * OUTPUT: boolean clear successful
-		 * ERROR:  -
-		 */
-		public function clear_cache() {
-			if ($this->db === null) {
-				return false;
-			}
-
-			$query = "truncate table %S";
-			return $this->db->query($query, "cache") !== false;
+			return true;
 		}
 	}
 ?>
